{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Import libraries"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "import ee\n",
    "import geemap"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Create an interactive map"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "Map = geemap.Map(center=[40, -100], zoom=4)"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Add Earth Engine Python script"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "# Add Earth Engine dataset\n",
    "image = ee.Image(\"USGS/SRTMGL1_003\")\n",
    "\n",
    "#  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n",
    "#  Chapter:      A2.5 Water Balance and Drought\n",
    "#  Checkpoint:   A25s3\n",
    "#  Author:       Ate Poortinga, Quyen Nguyen, Nyein Soe Thwal, Andréa Puzzi Nicolau\n",
    "#  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n",
    "\n",
    "# Import the Lower Mekong boundary.\n",
    "mekongBasin = ee.FeatureCollection(\n",
    "    'projects/gee-book/assets/A2-5/lowerMekongBasin')\n",
    "\n",
    "Map.centerObject(mekongBasin, 6)\n",
    "\n",
    "classStruct = {\n",
    "    'unknown': {\n",
    "        'number': 0,\n",
    "        'color': '6f6f6f'\n",
    "    },\n",
    "    'surface water': {\n",
    "        'number': 1,\n",
    "        'color': 'aec3d4'\n",
    "    },\n",
    "    'snow and ice': {\n",
    "        'number': 2,\n",
    "        'color': 'b1f9ff'\n",
    "    },\n",
    "    'mangroves': {\n",
    "        'number': 3,\n",
    "        'color': '111149'\n",
    "    },\n",
    "    'flooded forest': {\n",
    "        'number': 4,\n",
    "        'color': '287463'\n",
    "    },\n",
    "    'Deciduous forest': {\n",
    "        'number': 5,\n",
    "        'color': '152106'\n",
    "    },\n",
    "    'Orchard or plantation forest': {\n",
    "        'number': 6,\n",
    "        'color': 'c3aa69'\n",
    "    },\n",
    "    'evergreen Broadleaf': {\n",
    "        'number': 7,\n",
    "        'color': '7db087'\n",
    "    },\n",
    "    'mixed forest': {\n",
    "        'number': 8,\n",
    "        'color': '387242'\n",
    "    },\n",
    "    'urban and built up': {\n",
    "        'number': 9,\n",
    "        'color': 'cc0013'\n",
    "    },\n",
    "    'cropland': {\n",
    "        'number': 10,\n",
    "        'color': '8dc33b'\n",
    "    },\n",
    "    'rice': {\n",
    "        'number': 11,\n",
    "        'color': 'ffff00'\n",
    "    },\n",
    "    'mining': {\n",
    "        'number': 12,\n",
    "        'color': 'cec2a5'\n",
    "    },\n",
    "    'barren': {\n",
    "        'number': 13,\n",
    "        'color': '674c06'\n",
    "    },\n",
    "    'wetlands': {\n",
    "        'number': 14,\n",
    "        'color': '3bc3b2'\n",
    "    },\n",
    "    'grassland': {\n",
    "        'number': 15,\n",
    "        'color': 'f4a460'\n",
    "    },\n",
    "    'shrubland': {\n",
    "        'number': 16,\n",
    "        'color': '800080'\n",
    "    },\n",
    "    'aquaculture': {\n",
    "        'number': 17,\n",
    "        'color': '51768e'\n",
    "    }\n",
    "}\n",
    "\n",
    "# Function to get a list of ids (keys) from a structure.\n",
    "def getIds(struct):\n",
    "    return Object.keys(struct)\n",
    "\n",
    "\n",
    "# Function to replace spaces with underscores in a list of strings.\n",
    "def cleanList(list):\n",
    "\n",
    "def func_qat(name):\n",
    "        return name.replace(/\\s+/g, '_')\n",
    "\n",
    "    return list.map(func_qat)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "# Function to get a list of column values from a structure.\n",
    "def getList(struct, column):\n",
    "\n",
    "def func_iax(k):\n",
    "        value = struct[k][column]\n",
    "        return value\n",
    "\n",
    "    return Object.keys(struct).map(func_iax)\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "classNamesList = getIds(classStruct)\n",
    "probNames = cleanList(classNamesList)\n",
    "classNames = ee.List(classNamesList)\n",
    "classNumbers = getList(classStruct, 'number')\n",
    "paletteList = getList(classStruct, 'color')\n",
    "PALETTE = paletteList.join(',')\n",
    "\n",
    "# JSON dictionary that defines piechart colors based on the\n",
    "# landcover class palette.\n",
    "# https:#developers.google.com/chart/interactive/docs/gallery/piechart\n",
    "colors = []\n",
    "for i in range(0, paletteList.length, 1):\n",
    "    colors.push({\n",
    "        'color': '_'.replace('_', paletteList[i])\n",
    "    })\n",
    "\n",
    "\n",
    "# Set start and end years.\n",
    "startYear = 2010\n",
    "endYear = 2020\n",
    "\n",
    "# Create two date objects for start and end years.\n",
    "startDate = ee.Date.fromYMD(startYear, 1, 1)\n",
    "endDate = ee.Date.fromYMD(endYear + 1, 1, 1)\n",
    "\n",
    "# Make a list with years.\n",
    "years = ee.List.sequence(startYear, endYear)\n",
    "\n",
    "# Make a list with months.\n",
    "months = ee.List.sequence(1, 12)\n",
    "\n",
    "# Import the MOD16 dataset.\n",
    "mod16 = ee.ImageCollection('MODIS/006/MOD16A2').select('ET')\n",
    "\n",
    "# Filter for relevant time period.\n",
    "mod16 = mod16.filterDate(startDate, endDate)\n",
    "\n",
    "# Import the CHIRPS dataset.\n",
    "CHIRPS = ee.ImageCollection('UCSB-CHG/CHIRPS/PENTAD')\n",
    "\n",
    "# Filter for relevant time period.\n",
    "CHIRPS = CHIRPS.filterDate(startDate, endDate)\n",
    "\n",
    "landcover = ee.Image(\n",
    "        'projects/gee-book/assets/A2-5/RLCMSv3/Mekonglandcover2018') \\\n",
    "    .select('lc').clip(mekongBasin)\n",
    "\n",
    "lcVis = {\n",
    "    'palette': PALETTE,\n",
    "    'min': 0,\n",
    "    'max': classNamesList.length - 1\n",
    "}\n",
    "Map.addLayer(landcover, lcVis, '2018 Land Cover')\n",
    "\n",
    "# We apply a nested loop where we first iterate over\n",
    "# the relevant years and then iterate over the relevant\n",
    "# months. The function returns an image with P - ET\n",
    "# for each month. A flatten is applied to convert an\n",
    "# collection of collections into a single collection.\n",
    "waterBalance = ee.ImageCollection.fromImages(\n",
    "\n",
    "def func_pqv(y):\n",
    "        return months.map(function(m) {\n",
    "\n",
    "            P = CHIRPS.filter(ee.Filter \\\n",
    "                    .calendarRange(y, y, 'year')) \\\n",
    "                .filter(ee.Filter.calendarRange(m, m,\n",
    "                    'month')) \\\n",
    "                .sum()\n",
    "\n",
    "            ET = mod16.filter(ee.Filter \\\n",
    "                    .calendarRange(y, y, 'year')) \\\n",
    "                .filter(ee.Filter.calendarRange(m, m,\n",
    "                    'month')) \\\n",
    "                .sum() \\\n",
    "                .multiply(0.1)\n",
    "\n",
    "            wb = P.subtract(ET).rename('wb')\n",
    "\n",
    "            return wb.addBands(P).addBands(ET).set(\n",
    "                    'year', y) \\\n",
    "                .set('month', m) \\\n",
    "                .set('system:time_start', ee.Date \\\n",
    "                    .fromYMD(y, m, 1))\n",
    "        })\n",
    "\n",
    "    years.map(func_pqv\n",
    ").flatten()\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    ").flatten()\n",
    ")\n",
    "\n",
    "\n",
    "def func_ygw(m):\n",
    "    wbM = waterBalance.filter(ee.Filter.eq('month',\n",
    "            m)) \\\n",
    "        .mean() \\\n",
    "        .select('wb')\n",
    "    return ee.Image(wbM).set('month', m)\n",
    "\n",
    "wbMonth = ee.ImageCollection(months.map(func_ygw\n",
    "))\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "))\n",
    "\n",
    "# We convert the image collection to an image.\n",
    "wbMonth = wbMonth.toBands() \\\n",
    "    .select(['0_wb', '1_wb', '2_wb',\n",
    "            '3_wb', '4_wb', '5_wb',\n",
    "            '6_wb', '7_wb', '8_wb',\n",
    "            '9_wb', '10_wb', '11_wb'\n",
    "        ],\n",
    "        ['01_wb', '02_wb', '03_wb',\n",
    "            '04_wb', '05_wb', '06_wb',\n",
    "            '07_wb', '08_wb', '09_wb',\n",
    "            '10_wb', '11_wb', '12_wb'\n",
    "        ])\n",
    "\n",
    "# Select our classes of interest.\n",
    "classNumbers = ee.List([5, 7, 10, 11])\n",
    "\n",
    "# Apply reducer per land cover category.\n",
    "# We create binary map for each class and multiply by the monthly water balance.\n",
    "# The results are stored in a feature with other properties of the class.\n",
    "# The function returns a feature which are stored in a feature collection.\n",
    "\n",
    "def func_twh(nr):\n",
    "    lc = landcover.eq(ee.Number(nr))\n",
    "    wbM = wbMonth.multiply(lc)\n",
    "    wbSum = wbM.reduceRegion({\n",
    "        'reducer': ee.Reducer.sum(),\n",
    "        'geometry': mekongBasin,\n",
    "        'scale': 1000\n",
    "    })\n",
    "    return ee.Feature(None).set(wbSum) \\\n",
    "        .set('label', classNames.get(nr)) \\\n",
    "        .set('palette', paletteList[nr]) \\\n",
    "        .set('class_number', nr)\n",
    "\n",
    "lcFc = ee.FeatureCollection(classNumbers.map(func_twh\n",
    "))\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "\n",
    "))\n",
    "\n",
    "chart = ui.Chart.feature.byFeature({\n",
    "        'features': lcFc.select('[0-9][0-9]_wb|label'),\n",
    "        'xProperty': 'label',\n",
    "    }) \\\n",
    "    .setSeriesNames(['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',\n",
    "        'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'\n",
    "    ]) \\\n",
    "    .setChartType('ColumnChart') \\\n",
    "    .setOptions({\n",
    "        'title': 'water balance',\n",
    "        'hAxis': {\n",
    "            'title': 'landcover type',\n",
    "            'titleTextStyle': {\n",
    "                'italic': False,\n",
    "                'bold': True\n",
    "            }\n",
    "        },\n",
    "        'vAxis': {\n",
    "            'title': 'Water balance (mm)',\n",
    "            'titleTextStyle': {\n",
    "                'italic': False,\n",
    "                'bold': True\n",
    "            }\n",
    "        },\n",
    "        'colors': ['604791', '1d6b99', '39a8a7', '0f8755', '76b349',\n",
    "            'f0af07',\n",
    "            'e37d05', 'cf513e', '96356f', '724173', '9c4f97',\n",
    "            '696969'\n",
    "        ]\n",
    "    })\n",
    "\n",
    "print(chart)\n",
    "\n",
    "# Create the panel for the legend items.\n",
    "legend = ui.Panel({\n",
    "    'style': {\n",
    "        'position': 'bottom-left',\n",
    "        'padding': '8px 15px'\n",
    "    }\n",
    "})\n",
    "\n",
    "# Create and add the legend title.\n",
    "legendTitle = ui.Label({\n",
    "    'value': 'Legend',\n",
    "    'style': {\n",
    "        'fontWeight': 'bold',\n",
    "        'fontSize': '18px',\n",
    "        'margin': '0 0 4px 0',\n",
    "        'padding': '0'\n",
    "    }\n",
    "})\n",
    "\n",
    "# Creates and styles 1 row of the legend.\n",
    "def makeRow(color, name):\n",
    "    # Create the label that is actually the colored box.\n",
    "    colorBox = ui.Label({\n",
    "        'style': {\n",
    "            'backgroundColor': '#' + color,\n",
    "            # Use padding to give the box height and width.\n",
    "            'padding': '8px',\n",
    "            'margin': '0 0 4px 0'\n",
    "        }\n",
    "    })\n",
    "\n",
    "    # Create the label filled with the description text.\n",
    "    description = ui.Label({\n",
    "        'value': name,\n",
    "        'style': {\n",
    "            'margin': '0 0 4px 6px'\n",
    "        }\n",
    "    })\n",
    "\n",
    "    return ui.Panel({\n",
    "        'widgets': [colorBox, description],\n",
    "        'layout': ui.Panel.Layout.Flow('horizontal')\n",
    "    })\n",
    "\n",
    "\n",
    "legend.add(legendTitle)\n",
    "for i in range(0, classNamesList.length, 1):\n",
    "    legend.add(makeRow(paletteList[i], classNamesList[i]))\n",
    "\n",
    "\n",
    "# Add the legend to the map.\n",
    "Map.add(legend)\n",
    "\n",
    "# -----------------------------------------------------------------------------------\n",
    "# CHECKPOINT\n",
    "# -----------------------------------------------------------------------------------"
   ]
  },
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "## Display the interactive map"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": null,
   "metadata": {},
   "outputs": [],
   "source": [
    "Map"
   ]
  }
 ],
 "metadata": {
  "anaconda-cloud": {},
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.6.1"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
